

using System;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;


namespace GroupingPerf
{
	/// <summary>
	/// GroupingEngineFactory provides a trimmed down GridGroup
	/// which eliminates overhead of not needed preview rows, header and footer cells.
	/// </summary>
	public class GroupingEngineFactory : GridEngineFactoryBase
	{
		public override GridEngine CreateEngine()
		{
			return new GroupingEngine();
		}
	}

	public class GroupingEngine : GridEngine
	{
		public override Group CreateGroup(Section parent)
		{
			return new GroupingGroup(parent);
		}
		public override RecordRow CreateRecordRow(RecordRowsPart parent)
		{
			return new GroupingRecordRow(parent);
		}

	}


	public class GroupingGroup : Group, IGridGroupOptionsSource
	{
		public GroupingGroup(Section parent)
			: base(parent)
		{

		}

		protected override void OnInitializeSections(bool hasRecords, SortColumnDescriptorCollection fields)
		{
			this.Sections.Add(this.ParentTableDescriptor.CreateCaptionSection(this));
			if (hasRecords)
				this.Sections.Add(this.ParentTableDescriptor.CreateRecordsDetails(this, fields));
			else
				this.Sections.Add(this.ParentTableDescriptor.CreateGroupsDetails(this, fields));
			
			this.Sections.Add(this.ParentTableDescriptor.CreateSummarySection(this));
			
		}

		
		public override bool IsChildVisible(Element el)
		{	
			//Record obj = el.GetRecord();
			//int zip = Convert.ToInt32(obj.GetValue("id"));
			
			if (el is CaptionSection)
			{
				return true;
			}
			else if (el is DetailsSection)
			{	
				//				DataRow data = (DataRow)el.GetRecord();
				//				int zipcode = Convert.ToInt32(data);

				//return this.IsExpanded;
				return this.IsExpanded;		 // if it returns false then details wont show.
			}	
			else if (el is GridSummarySection)
			{
				return this.IsExpanded;
			}
				
			return true;
		}

		protected override bool IsGroupVisible() 
		{
			return true;
		}
		
//		public override bool ShouldShowCaption()
//		{
//			return true;
//		}
		 
		#region IGridGroupOptionsSource Members

		GridGroupOptionsStyleInfo groupOptions;

		public void RaiseGroupOptionsChanging(GridGroupOptionsChangedEventArgs e)
		{
		}

		public void RaiseGroupOptionsChanged(GridGroupOptionsChangedEventArgs e)
		{
			this.InvalidateCounterTopDown(true);
		}

		public bool HasGroupOptions
		{
			get
			{
				return groupOptions != null;
			}
		}

		/// <override/>
		public new GridEngine Engine
		{
			get
			{
				return (GridEngine) base.Engine;
			}
		}

		/// <summary>
		/// Lets you control the look of inner groups like whether the Caption Row is visible, or what CaptionText is.
		/// </summary>
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
		[Browsable(true)]
		public GridGroupOptionsStyleInfo GroupOptions
		{
			get
			{
				if (groupOptions == null)
				{
					GridGroupOptionsType t = GridGroupOptionsType.Groups;
					if (this.IsTopLevelGroup)
						t = GridGroupOptionsType.TopLevelGroup;
					else //if (this is ChildTable)
						t = GridGroupOptionsType.ChildTable;

					groupOptions = new GridGroupOptionsStyleInfo(new GridGroupOptionsStyleInfoIdentity((IGridGroupOptionsSource) this, t));
				}
				return groupOptions;
			}
			set
			{
				GroupOptions.CopyFrom(value);
			}
		}

		public IGridGroupOptionsSource GetParentGroupOptionsSource()
		{
			if (this.CategoryColumns.Count > 0)
			{
				GridColumnDescriptor column = ((GridTableDescriptor) this.ParentTableDescriptor).Columns[this.CategoryColumns[0].Name];
				if (column != null)
					return column;
			}
			return (IGridGroupOptionsSource) this.ParentTableDescriptor;
		}

		#endregion
	}

	
	public class GroupingRecordRow : GridRecordRow, IGridRowHeight
	{
		int rowHeight = -1;

		/// <summary>
		/// Initializes a new object in the specifed record part.
		/// </summary>
		/// <param name="parent">The parent element.</param>
		public GroupingRecordRow(RecordRowsPart parent)
			: base(parent)
		{
		}
		#region IGridRowHeight Members

		/// <summary>
		/// Determines if elements supports storing row heights 
		/// </summary>
		/// <returns></returns>
		public bool SupportsRowHeight()
		{
			return true;
		}

		/// <summary>
		/// The row height 
		/// </summary>
		public int RowHeight
		{
			get
			{
				return rowHeight;
			}
			set
			{
				if (rowHeight != value)
				{
					rowHeight = value;
					this.InvalidateCounterBottomUp();
				}
			}
		}

		/// <summary>
		/// Checks if row height was modified or if default setting should be used.
		/// </summary>
		public bool HasRowHeight
		{
			get
			{
				return rowHeight != -1;
			}
		}

		#endregion

		/// <summary>
		/// This is where the row height then gets integrated with the engine
		/// YAmount Counter logic.
		/// </summary>
		/// <returns></returns>
		public override double GetYAmountCount()
		{
			// Note: whenever the value that is returned by GetYAmountCount changes
			// make sure you call InvalidateCounterBottomUp so that the engine
			// is aware of the change and counters are recalculated. See
			// the RowHeight setter. 
			return rowHeight != -1 ? rowHeight : base.GetYAmountCount();
		}

	}

}
